@@ -15,6 +15,7 @@
 #include <gdk/gdkx.h>
 #include <stdlib.h>
 
+#include "gduutils.h"
 #include "gduapplication.h"
 #include "gduwindow.h"
 #include "gducreatefilesystemwidget.h"
@@ -187,7 +188,13 @@ update (GduCreateFilesystemWidget *widget)
   const gchar *passphrase = NULL;
   const gchar *id;
 
-  name = gtk_entry_get_text (GTK_ENTRY (widget->name_entry));
+  gtk_entry_set_icon_from_icon_name (GTK_ENTRY (widget->confirm_passphrase_entry),
+                                     GTK_ENTRY_ICON_SECONDARY,
+                                     NULL);
+  gtk_entry_set_icon_tooltip_text (GTK_ENTRY (widget->confirm_passphrase_entry),
+                                   GTK_ENTRY_ICON_SECONDARY,
+                                   NULL);
+
   passphrase = gtk_entry_get_text (GTK_ENTRY (widget->passphrase_entry));
 
   id = gtk_combo_box_get_active_id (GTK_COMBO_BOX (widget->type_combobox));
@@ -219,6 +226,15 @@ update (GduCreateFilesystemWidget *widget)
             {
               has_info = TRUE;
             }
+          else if (strlen (gtk_entry_get_text (GTK_ENTRY (widget->confirm_passphrase_entry))) > 0)
+            {
+              gtk_entry_set_icon_from_icon_name (GTK_ENTRY (widget->confirm_passphrase_entry),
+                                                 GTK_ENTRY_ICON_SECONDARY,
+                                                 "dialog-warning-symbolic");
+              gtk_entry_set_icon_tooltip_text (GTK_ENTRY (widget->confirm_passphrase_entry),
+                                               GTK_ENTRY_ICON_SECONDARY,
+                                               _("The passphrases do not match"));
+            }
         }
       gdu_password_strength_widget_set_password (GDU_PASSWORD_STRENGTH_WIDGET (widget->passphrase_strengh_widget),
                                                  gtk_entry_get_text (GTK_ENTRY (widget->passphrase_entry)));
@@ -244,6 +260,10 @@ update (GduCreateFilesystemWidget *widget)
       has_info = TRUE;
     }
 
+  _gtk_entry_buffer_truncate_bytes (gtk_entry_get_buffer (GTK_ENTRY (widget->name_entry)),
+                                    gdu_utils_get_max_label_length (fstype));
+  name = gtk_entry_get_text (GTK_ENTRY (widget->name_entry));
+
   if (show_name_widgets)
     {
       gtk_widget_show (widget->name_label);
@@ -556,7 +576,11 @@ gdu_create_filesystem_widget_constructed (GObject *object)
                       TRUE, TRUE, 0);
 
   /* reparent and nuke the dummy window */
-  gtk_widget_reparent (widget->grid, GTK_WIDGET (widget));
+  g_object_ref (widget->grid);
+  gtk_container_remove (GTK_CONTAINER (dummy_window), widget->grid);
+  gtk_container_add (GTK_CONTAINER (widget), widget->grid);
+  g_object_unref (widget->grid);
+
   gtk_widget_destroy (dummy_window);
 
   populate (widget);
